<!--- This work is licensed under the Creative Commons Attribution-ShareAlike 2.5 License License. 
To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/2.5/ or send a letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.

In order to comply with the attribution clause of the Creative Commons License, 
please do not remove this and the "Developed by Blue Instant" notice

Use this application at your own risk

<rdf:RDF xmlns="http://web.resource.org/cc/" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
		<Work rdf:about="">
			<license rdf:resource="http://creativecommons.org/licenses/by-sa/2.5/" />
	<dc:title>File Explorer</dc:title>
	<dc:date>2006</dc:date>
	<dc:creator><Agent><dc:title>Laura Arguello and Nahuel Foronda</dc:title></Agent></dc:creator>
	<dc:rights><Agent><dc:title>Laura Arguello and Nahuel Foronda</dc:title></Agent></dc:rights>
	<dc:source rdf:resource="http://www.asfusion.com/projects/fileexplorer" />
		</Work>
		<License rdf:about="http://creativecommons.org/licenses/by-sa/2.5/"><permits rdf:resource="http://web.resource.org/cc/Reproduction"/><permits rdf:resource="http://web.resource.org/cc/Distribution"/><requires rdf:resource="http://web.resource.org/cc/Notice"/><requires rdf:resource="http://web.resource.org/cc/Attribution"/><permits rdf:resource="http://web.resource.org/cc/DerivativeWorks"/><requires rdf:resource="http://web.resource.org/cc/ShareAlike"/></License></rdf:RDF>
 --->

<cfcomponent name="FTPManager" access="public" description="Component that manages all directory and file operations on an FTP server. ">
<!--- Warning: this component has not been much tested, use as your own responsability --->

<cfset variables.messages = structnew()/>
<cfset variables.messages["FolderNotExist"] = "Folder does not exist"/>
<cfset variables.messages["FileNotExist"] = "File does not exist"/>

 <!--- :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: --->
<cffunction name="init" output="false" returntype="any" hint="instantiates an object of this class" access="public">
	<cfargument name="server" required="true" type="any">
	<cfargument name="username" required="true" type="string">
	<cfargument name="password" required="true" type="string">
	<cfargument name="extensions" required="false" default="*" type="string">
	<cfargument name="passive" required="false" type="boolean" default="true">
	<cfargument name="port" required="false" type="numeric" default="21">
		
		<cfset variables.server = arguments.server />
		<cfset variables.username = arguments.username />
		<cfset variables.password = arguments.password />
		<cfset variables.passive = arguments.passive />
		<cfset variables.port = arguments.port />
		<cfset variables.extensions = arguments.extensions />
		
		<!--- get the system file separator --->
		<cfset variables.fileSeparator = "/" />
		
		<cfreturn this />
</cffunction>

<!--- :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: --->
<cffunction name="getFiles" output="false" description="Returns the list of files in given path - starting from root path" 
					access="public" returntype="struct">
	<cfargument name="path" required="false" type="string" default=""/>
	
		<cfset var files = ""/>
		<cfset var allElements = ""/>
		<cfset var result = structnew()/>
		<cfset result["status"] = true />
		
		<cftry>
			<cfftp server="#variables.server#"  username="#variables.username#" 
				password="#variables.password#" port="#variables.port#" passive="#variables.passive#" stoponerror="Yes"
				action="listdir" name="allElements" directory="#arguments.path#">
			
				<cfquery name="files" dbtype="query">
					SELECT lastmodified as DateLastModified, Path, length as Size, Name
					FROM allElements
					WHERE isDirectory = 'NO'
				</cfquery>	
				
				<cfset result["files"] = files />
				<cfset result["message"] = files.recordcount & " files found" />
				
			<cfcatch type="Any">
					<cfset result["files"] = queryNew("Name") />
					<cfset result["status"] = false />
					<cfset result["message"] = cfcatch.message  />
				</cfcatch>
			</cftry>
		
		<cfreturn result />
		
</cffunction>


<!--- :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: --->
<cffunction name="getDirectories" output="false" description="Returns the list of directories in given path - starting from root path" 
					access="public" returntype="struct">
	<cfargument name="path" required="false" type="string" default=""/>
	<cfargument name="addHasDirs" required="false" type="boolean" default="true"/>
	
		<cfset var dirs = ""/>
		<cfset var allElements = ""/>
		<cfset var subdirs = ""/>
		<cfset var result = structnew()/>
		<cfset result["status"] = true />
		
		<cftry>

			<cfftp server="#variables.server#"  username="#variables.username#" 
				password="#variables.password#" port="#variables.port#" passive="#variables.passive#" stoponerror="Yes"
				action="listdir" name="allElements" directory="#arguments.path#">
		
				<cfquery name="dirs" dbtype="query">
					SELECT lastmodified as DateLastModified, Path, length as Size, Name <cfif arguments.addHasDirs>, 0 as hasDirs</cfif>
					FROM allElements
					WHERE isDirectory = 'YES'
				</cfquery>
		
				<cfif arguments.addHasDirs>
					<!--- recurse once to add whether it has sub dirs --->
					<cfoutput query="dirs">
						<cfset subdirs = getDirectories(dirs.Path,false).directories />
						<cfif subdirs.recordcount>
							<cfset querysetcell(dirs,"hasDirs",1,currentrow)/>
						</cfif>
					</cfoutput>
				</cfif>
				
				<cfset result["message"] = dirs.recordcount & " sub-directories found" />
				<cfset result["directories"] = dirs/>
								
			<cfcatch type="Any">
					<cfset result["directories"] = queryNew("Name") />
					<cfset result["status"] = false />
					<cfset result["message"] = cfcatch.message />
				</cfcatch>
			</cftry>
		
		<cfreturn result />
		
</cffunction>

<!--- :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: --->
<cffunction name="createFolder" output="false" description="Creates a folder under the given path" 
					access="remote" returntype="struct">
	<cfargument name="path" required="false" type="string" default=""/>
	<cfargument name="name" required="false" type="string" default=""/>	

		<cfset var exists = ""/>
		<cfset var newdir = arguments.path & variables.fileSeparator & arguments.name />
		<cfset var result = structnew()/>
		<cfset var connection = ""/>
		<cfset result["status"] = false />
		
		<cftry>
			<cfftp connection="connection" server="#variables.server#"  username="#variables.username#" 
				password="#variables.password#" port="#variables.port#" passive="#variables.passive#" stoponerror="Yes" 
				action="open">
			
			<cfftp action="EXISTSDIR" directory="#newdir#" connection="connection" result="exists">
			
			<cfif NOT exists.returnValue>
				<cftry>
					<cfftp action="CREATEDIR" directory="#newdir#" connection="connection">
					
					<cfset result["completepath"] = "#newdir#" />
					<cfset result["status"] = true />
					<cfset result["message"] = 'Folder "#arguments.name#" created' />
					
					<cfcatch type="Any">
						<cfset result["message"] = cfcatch.message />
					</cfcatch>
				</cftry>
				
			<cfelse>
				<!--- this folder already exists --->	
				<cfset result["message"] = 'Folder "#arguments.name#" already exists' />
			</cfif>
			
		<cfcatch type="Any">
				<cfset result["message"] = cfcatch.message/>
			</cfcatch>
		</cftry>
		
		<cfftp action="close" connection="connection">
		
		<cfreturn result/>
		
</cffunction>

<!--- :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: --->
<cffunction name="removeFolder" output="false" description="Deletes a folder" 
					access="remote" returntype="struct">
	<cfargument name="path" required="false" type="string" default=""/>
		<!--- @TODO: check that base path is not path? 
		@TODO: recurse to delete subdirs
		--->
		<cfset var exists = ""/>
		<cfset var result = structnew()/>
		<cfset var connection = ""/>
		<cfset result["status"] = false />
		
		<cftry>
			<cfftp connection="connection" server="#variables.server#"  username="#variables.username#" 
				password="#variables.password#" port="#variables.port#" passive="#variables.passive#" stoponerror="Yes" 
				action="open">
			<cfftp action="EXISTSDIR" directory="#arguments.path#" connection="connection" result="exists">
			
			<cfif exists.returnValue>
				<cfftp action="REMOVEDIR" directory="#arguments.path#" connection="connection">
			
				<cfset result["status"] = true />
				<cfset result["message"] = 'Folder "#listlast(arguments.path,variables.fileSeparator)#" and its contents deleted' />
			<cfelse>
				<cfset result["message"] = "Directory does not exist" />
			</cfif>
			<cfcatch type="Any">
				<cfset result["message"] = cfcatch.message />
			</cfcatch>
		</cftry>
		
		<cfftp action="close" connection="connection">
		<cfreturn result/>
		
</cffunction>


<!--- :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: --->
<cffunction name="renameFolder" output="false" description="Renames a folder" 
					access="remote" returntype="struct">
	<cfargument name="path" required="false" type="string" default=""/>
	<cfargument name="name" required="false" type="string" default=""/>	
	<cfargument name="newname" required="false" type="string" default=""/>	

		<cfset var exists = ""/>
		<cfset var connection = ""/>
		<cfset var result = structnew()/>
		<cfset result["status"] = false />
		<cfset result["oldpath"] = "#arguments.path##variables.fileSeparator##arguments.name#" />
		<cfset result["renamedpath"] = "#arguments.path##variables.fileSeparator##arguments.newname#" />
		
	<cftry>
		<cfftp connection="connection" server="#variables.server#"  username="#variables.username#" 
				password="#variables.password#" port="#variables.port#" passive="#variables.passive#" stoponerror="Yes" 
				action="open">
		<cfftp action="EXISTSDIR" directory="#result['oldpath']#" connection="connection" result="exists">
			
		<cfif exists.returnValue>
		
			<cfftp action="EXISTSDIR" directory="#result['renamedpath']#" connection="connection" result="exists">
			
			<cfif NOT exists.returnValue>
				<cfftp action="RENAME" stoponerror="Yes" existing="#result['oldpath']#" new="#result['renamedpath']#" connection="connection">
												
						<cfset result["status"] = true />					
						<cfset result["message"] = 'Folder "#arguments.name#" renamed to "#arguments.newname#"' />			
						
				<cfelse>
					<cfset result["message"] = 'A folder called "#arguments.newname#" already exists, cannot rename "#arguments.name#"' />
				</cfif>
			
			<cfelse>
					<cfset result["message"] = "Directory does not exist" />
			</cfif>
			
			<cfcatch type="Any">
				<cfset result["message"] = cfcatch.message />
			</cfcatch>
		</cftry>
		
		<cfftp action="close" connection="connection">
		<cfreturn result/>
		
</cffunction>

<!--- :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: --->
<cffunction name="removeFile" output="false" description="Deletes a file" 
					access="remote" returntype="struct">
	<cfargument name="path" required="false" type="string" default=""/>
	<cfargument name="name" required="false" type="string" default=""/>	

		<cfset var result = structnew()/>
		<cfset var exists = ""/>
		<cfset var connection = ""/>
		<cfset var completePath = "#arguments.path##variables.fileSeparator##arguments.name#"/>
		<cfset result["status"] = false />
		
		<cftry>
			<cfftp connection="connection" server="#variables.server#"  username="#variables.username#" 
				password="#variables.password#" port="#variables.port#" passive="#variables.passive#" stoponerror="Yes" 
				action="open">
			<cfftp action="EXISTSFILE" remotefile="#completePath#" connection="connection" result="exists">
			
			<cfif exists.returnValue>
		
				<cfftp action="remove" item="#completePath#" connection="connection">
											
				<cfset result["status"] = true />
				<cfset result["message"] = 'File "#arguments.name#" deleted' />		
			
			<cfelse>			
				<cfset result["message"] = variables.messages["FileNotExist"] />
			</cfif>	
		
			<cfcatch type="Any">
				<cfset result["message"] = cfcatch.message />
			</cfcatch>
		</cftry>
		
		<cfftp action="close" connection="connection">
		<cfreturn result/>
		
</cffunction>

<!--- :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: --->
<cffunction name="renameFile" output="false" description="Renames a file" 
					access="remote" returntype="struct">
	<cfargument name="path" required="false" type="string" default=""/>
	<cfargument name="name" required="false" type="string" default=""/>	
	<cfargument name="newname" required="false" type="string" default=""/>	

		<cfset var result = structnew()/>
		<cfset var exists = ""/>
		<cfset var connection = ""/>
		<cfset var oldpath = "#arguments.path##variables.fileSeparator##arguments.name#"/>
		<cfset var newpath = "#arguments.path##variables.fileSeparator##arguments.newname#"/>
		<cfset result["status"] = false />
		
		<cftry>
		<cfftp connection="connection" server="#variables.server#"  username="#variables.username#" 
				password="#variables.password#" port="#variables.port#" passive="#variables.passive#" stoponerror="Yes" 
				action="open">
		<cfftp action="EXISTSFILE" remotefile="#oldpath#" connection="connection" result="exists">
			
		<cfif exists.returnValue>
		
			<cfftp action="EXISTSFILE" remotefile="#newpath#" connection="connection" result="exists">
			
			<cfif NOT exists.returnValue>
				<cfftp action="RENAME" stoponerror="Yes" existing="#oldpath#" new="#newpath#" connection="connection">
												
						<cfset result["status"] = true />					
						<cfset result["message"] = 'File "#arguments.name#" has been renamed to "#arguments.newname#"' />	
						
				<cfelse>
					<cfset result["message"] = 'A file called "#arguments.newname#" already exists, cannot rename "#arguments.name#"' />
				</cfif>
			
			<cfelse>
				<cfset result["message"] = variables.messages["FileNotExist"] />
			</cfif>
			
			<cfcatch type="Any">
				<cfset result["message"] = cfcatch.message />
			</cfcatch>
		</cftry>
		
		<cfftp action="close" connection="connection">
		<cfreturn result/>
	
</cffunction>


<!--- :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: --->
<cffunction name="uploadFile" output="false" description="Uploads a file in the given folder" 
					access="public" returntype="void">
	<cfargument name="filefield" required="true" />
	<cfargument name="path" required="false" type="string" default=""/>	
	<cfargument name="filename" type="string" required="true" />

	<cfset var fileResults = ""/>
	<cfset var newfile = ""/>
	
		<cfif len(arguments.filename) AND isAllowedExtension(arguments.filename)>
		<!--- store it in a temp directory --->
			<cffile action="UPLOAD" filefield="#arguments.filefield#" destination="#GetTempDirectory()#" result="fileResults" nameconflict="MAKEUNIQUE">
			<cfset newfile = fileResults.serverFile />
			
			<!--- save in ftp --->
			<cfftp server="#variables.server#"  username="#variables.username#" 
				password="#variables.password#" port="#variables.port#" passive="#variables.passive#" stoponerror="Yes"
				action="putFile" localfile="#GetTempDirectory()##newfile#" remotefile="#arguments.path##variables.fileSeparator##arguments.filename#">
			
		<cfelse>			
			<cfthrow message='"#getExtension(arguments.filename)#" is not a valid extension.' />
		</cfif>
		
</cffunction>


<!--- :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: --->
<cffunction name="getFile" output="false" description="Downloads a file" 
					access="public" returntype="string">
	<cfargument name="path" required="false" type="string" default=""/>	
	<cfargument name="filename" type="string" required="true" />
	<cfset var filecontent = ""/>
		
		<!--- get it from the ftp --->
		<cfftp server="#variables.server#"  username="#variables.username#" 
				password="#variables.password#" port="#variables.port#" passive="#variables.passive#" stoponerror="Yes"
				action="getFile" localfile="#GetTempDirectory()##filename#" remotefile="#arguments.path##variables.fileSeparator##arguments.filename#"
 				 failifexists="No">
		
				<cffile action="READ" file="#GetTempDirectory()##filename#" variable="filecontent">
	
	<cfreturn filecontent />
</cffunction>


<!--- :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: --->
<cffunction name="isAllowedExtension" output="false" description="Checks whether this is not an allowed extension" 
					access="private" returntype="boolean">
	<cfargument name="filename" type="string" required="true" />
		
		<cfif variables.extensions EQ "*">
			<cfreturn true />
		</cfif>
		<cfreturn listfind(variables.extensions,getExtension(arguments.filename)) />
		
</cffunction>

<!--- :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: --->
<cffunction name="getExtension" output="false" description="Returns the extension of a file" 
					access="private" returntype="string">
	<cfargument name="name" type="string" required="true" />

		<cfif find(".",arguments.name)>
			<cfreturn listlast(arguments.name,".")/>
		<cfelse>
			<cfreturn ""/>
		</cfif>
		
</cffunction>

</cfcomponent>